dojo.provide("com.ibm.pb.io.ResourceCache");

/**
 * Manages a cache of resources such that IO calls can be resolved directly
 * by means of a plugin to the DynamicResolver service.
 */
dojo.declare("com.ibm.pb.io.ResourceCache", null, {
	_mpExp: /mashup:huffman/,
	PLUGIN_ID: "com.ibm.pb.io.ResourceCache",
	DYNRES: com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.io.DynamicResolver.SERVICE_NAME),
	constructor: function() {
		this._cacheKeys = [];
		this._cache = {};
	},
	_matcher: function(url){
		url = url ? url.toString() : ""; // normalize
		var exp = new RegExp(this._cacheKeys.join("|")),
			m;
		url = url.replace(/\/(?:\/)+/,"/"); // remove multiple, concurrent slashes
		if(!url.match(this._mpExp)) { // don't match multipart requests
			return ((m = url.match(exp)) ? m[0] : false);	// String, cacheKey
		}
		return false;
	},
	/**
	 * Sets the value of the cache entry associated with key.
	 */
	set: function(key, val) {
		if(!key){return;}
		if(!this._cache[key]) {
			if(this._cacheKeys.length === 0) {
				var me = this;	// lazy registration
				this.DYNRES.register(this.PLUGIN_ID, 
					function(url){return me._matcher(url);},
					function(url, cacheKey) {return me.get(cacheKey);}
				);
			}
			this._cacheKeys.push(key);
		}
		if(!val) { // clearing a value
			for(var i=0;i<this._cacheKeys.length;i++){
				if(this._cacheKeys[i] === key) {
					this._cacheKeys.splice(i,1);
					break;
				}
			}
			if(this._cacheKeys.length === 0) {
				this.DYNRES.unregister(this.PLUGIN_ID);
			}
		}
		this._cache[key] = val;
	},
	get: function(key) {
		return this._cache[key];
	}
});

com.ibm.pb.io.RESOURCE_CACHE = new com.ibm.pb.io.ResourceCache();